import os
from flask import Flask, request, abort
from flask_migrate import Migrate
from stumpf.models import db
from stumpf.views import artifact_bp, task_bp, task_def_bp, video_bp


def before_request_auth(*args, **kwargs):
    if request.headers.get('X-Stumpf-Token') != os.getenv('X_STUMPF_TOKEN'):
        abort(403)


def create_app():
    app = Flask(__name__)
    app.config['SQLALCHEMY_DATABASE_URI'] = os.getenv('STUMPF_POSTGRES_URI')
    db.init_app(app)
    Migrate(app, db)
    app.register_blueprint(artifact_bp, url_prefix='/api/v1')
    app.register_blueprint(task_bp, url_prefix='/api/v1')
    app.register_blueprint(task_def_bp, url_prefix='/api/v1')
    app.register_blueprint(video_bp, url_prefix='/api/v1')

    app.before_request(before_request_auth)

    return app
