from datetime import datetime
from flask_sqlalchemy import SQLAlchemy
from marshmallow import Schema, fields, post_load
from sqlalchemy.dialects.postgresql import UUID
import uuid


db = SQLAlchemy()


# todo figure out timezone in created

class BaseSchema(Schema):
    id = fields.UUID(read_only=True)
    created = fields.DateTime(read_only=True)


class Video(db.Model):
    id = db.Column(UUID, primary_key=True, default=uuid.uuid4)
    created = db.Column(db.DateTime, nullable=False, default=datetime.utcnow)
    path = db.Column(db.String, unique=True, nullable=False)
    name = db.Column(db.String, unique=True, nullable=False)
    frame_rate = db.Column(db.Float, nullable=False)
    length = db.Column(db.Float, nullable=False)
    width = db.Column(db.Integer, nullable=False)
    height = db.Column(db.Integer, nullable=False)


class VideoSchema(BaseSchema):
    path = fields.Str(required=True)
    name = fields.Str(required=True)
    frame_rate = fields.Float(required=True)
    length = fields.Float(required=True)
    width = fields.Integer(required=True)
    height = fields.Integer(required=True)

    @post_load
    def make_video(self, data):
        return Video(**data)


class TaskDef(db.Model):
    id = db.Column(UUID, primary_key=True, default=uuid.uuid4)
    created = db.Column(db.DateTime, nullable=False, default=datetime.utcnow)
    name = db.Column(db.String, unique=True, nullable=False)
    type_ = db.Column(db.String, nullable=False)  # one of: HIT, one-off


class TaskDefSchema(BaseSchema):
    name = fields.Str(required=True)
    type_ = fields.Str(required=True)

    @post_load
    def make_task_def(self, data):
        return TaskDef(**data)


class Task(db.Model):
    id = db.Column(UUID, primary_key=True, default=uuid.uuid4)
    created = db.Column(db.DateTime, nullable=False, default=datetime.utcnow)
    task_def_id = db.Column(UUID, db.ForeignKey(TaskDef.id), nullable=False)
    video_id = db.Column(UUID, db.ForeignKey('video.id'), nullable=False)
    completed = db.Column(db.DateTime, nullable=True, default=None)

    video = db.relationship('Video', backref=db.backref('video', lazy=True))
    taskdef = db.relationship('TaskDef', backref=db.backref('taskdef', lazy=True))


class TaskSchema(BaseSchema):
    task_def_id = fields.Str(required=True)
    video_id = fields.Str(required=True)
    completed = fields.DateTime()

    @post_load
    def make_task(self, data):
        return Task(**data)

class UpdateableTaskSchema(Schema):
    completed = fields.DateTime()


class Artifact(db.Model):
    id = db.Column(UUID, primary_key=True, default=uuid.uuid4)
    created = db.Column(db.DateTime, nullable=False, default=datetime.utcnow)
    task_id = db.Column(UUID, db.ForeignKey('task.id'), nullable=False)
    path = db.Column(db.String, unique=True, nullable=False)

    task = db.relationship('Task', backref=db.backref('task', lazy=True))


class ArtifactSchema(BaseSchema):
    task_id = fields.Str(required=True)
    path = fields.Str(required=True)

    @post_load
    def make_artifact(self, data):
        return Artifact(**data)
