from flask import Blueprint, jsonify, request
from marshmallow import ValidationError
from stumpf.models import (db, Video, VideoSchema, TaskDef, TaskDefSchema, Task,
                           TaskSchema, UpdateableTaskSchema, Artifact, ArtifactSchema)
import uuid

artifact_bp = Blueprint('artifact', __name__)
video_bp = Blueprint('video', __name__)
task_bp = Blueprint('task', __name__)
task_def_bp = Blueprint('task-definition', __name__)


@video_bp.route('/video', methods=['POST'])
def create_video():
    schema = VideoSchema()

    try:
        video = schema.load(request.json)
        # todo - stop setting id this way
        video.id = str(uuid.uuid4())

        # todo, figure out how to do this with marshmallow
        if Video.query.filter_by(path=video.path).first():
            return jsonify({
                'error': 'Video path already exists'
            }), 409

        db.session.add(video)
        db.session.commit()
        return jsonify(schema.dump(video)), 201
    except ValidationError as e:
        return jsonify(e.messages), 400
    except Exception as e:
        db.session.rollback()
        raise

    return jsonify(schema.dump(video))


@video_bp.route('/video', methods=['GET'])
def list_videos():
    schema = VideoSchema()
    return jsonify(schema.dump(Video.query.all(), many=True))


@task_def_bp.route('/task-definition', methods=['POST'])
def create_task_definition():
    schema = TaskDefSchema()

    try:
        task_def = schema.load(request.json)
        # todo - stop setting id this way
        task_def.id = str(uuid.uuid4())

        # todo, figure out how to do this with marshmallow
        if TaskDef.query.filter_by(name=task_def.name).first():
            return jsonify({
                'error': 'Task definition with that name already exists'
            }), 409

        db.session.add(task_def)
        db.session.commit()
        return jsonify(schema.dump(task_def)), 201
    except ValidationError as e:
        return jsonify(e.messages), 400
    except Exception as e:
        db.session.rollback()
        raise

    return jsonify(schema.dump(task_def))


@task_def_bp.route('/task-definition', methods=['GET'])
def list_task_defs():
    schema = TaskDefSchema()
    return jsonify(schema.dump(TaskDef.query.all(), many=True))


@task_def_bp.route('/task', methods=['GET'])
def list_tasks():
    schema = TaskSchema()
    return jsonify(schema.dump(Task.query.all(), many=True))


@task_bp.route('/task', methods=['POST'])
def create_task():
    schema = TaskSchema()

    try:
        task = schema.load(request.json)
        # todo - stop setting id this way
        task.id = str(uuid.uuid4())

        # todo, marshmallow
        if not TaskDef.query.get(task.task_def_id):
            return jsonify({
                'error': 'No such task definition.'
            }), 400
        elif not Video.query.get(task.video_id):
            return jsonify({
                'error': 'No such video.'
            }), 400

        db.session.add(task)
        db.session.commit()
        return jsonify(schema.dump(task)), 201
    except ValidationError as e:
        return jsonify(e.messages), 400
    except Exception as e:
        db.session.rollback()
        raise

    return jsonify(schema.dump(task))


# todo put vs patch
@task_bp.route('/task/<task_id>', methods=['PUT'])
def update_task(task_id):
    task = Task.query.get_or_404(task_id)

    if task.completed is not None:
        return jsonify({
            'error': 'Task is already completed'
        }), 409

    schema = UpdateableTaskSchema()

    try:
        task_update_attrs = schema.load(request.json)

        for attr, value in task_update_attrs.items():
            setattr(task, attr, value)

        db.session.add(task)
        db.session.commit()
        return jsonify(schema.dump(task)), 201
    except ValidationError as e:
        return jsonify(e.messages), 400
    except Exception as e:
        db.session.rollback()
        raise

    return jsonify(schema.dump(task))

@artifact_bp.route('/artifact', methods=['POST'])
def create_artifact():
    schema = ArtifactSchema()

    try:
        artifact = schema.load(request.json)
        # todo - stop setting id this way
        artifact.id = str(uuid.uuid4())

        # todo, figure out how to do this with marshmallow
        task = Task.query.get(artifact.task_id)
        if not task:
            return jsonify({
                'error': 'Task does not exist'
            }), 400
        elif task.completed is not None:
            return jsonify({
                'error': 'Task has already been completed'
            }), 409

        db.session.add(artifact)
        db.session.commit()
        return jsonify(schema.dump(artifact)), 201
    except ValidationError as e:
        return jsonify(e.messages), 400
    except Exception as e:
        db.session.rollback()
        raise

    return jsonify(schema.dump(artifact))


@artifact_bp.route('/artifact/<task_id>', methods=['GET'])
def list_artifacts(task_id):
    Task.query.get_or_404(task_id)
    schema = ArtifactSchema()
    return jsonify(schema.dump(Artifact.query.filter_by(task_id=task_id).all(), many=True))
